//******************************************************************
void CheckPins(uint8_t HighPin, uint8_t LowPin, uint8_t TristatePin)
  {
  /*
  Funktion zum Testen der Eigenschaften des Bauteils bei der angegebenen Pin-Belegung
  Parameter:
  HighPin: Pin, der anfangs auf positives Potenzial gelegt wird
  LowPin: Pin, der anfangs auf negatives Potenzial gelegt wird
  TristatePin: Pin, der anfangs offen gelassen wird

  Im Testverlauf wird TristatePin natrlich auch positiv oder negativ geschaltet.
*/
  struct {
     unsigned int lp_otr;
     unsigned int hp1;
     unsigned int hp2;
     unsigned int hp3;
     unsigned int lp1;
     unsigned int lp2;
     unsigned int tp1;
     unsigned int tp2;
  }adc;
  uint8_t LoPinRL, LoPinRH;	// Schaltbefehl fr LowPin R_L / R_H
  uint8_t TriPinRL, TriPinRH;	// Schaltbefehl fr TristatePin R_L / R_H
  uint8_t HiPinRL, HiPinRH;	// Schaltbefehl fr HighPin R_L / R_H
  uint8_t HiADCp, LoADCp;		// Schaltbefehl fr ADC-Port High-Pin / Low-Pin
  uint8_t HiADCm, LoADCm;		// Schaltbefehl fr ADC-DDR High-Pin / Low-Pin
  uint8_t PinMSK;
  uint8_t ii;			// Hilfsvariable

#ifdef KOLLEKTOR_SCHALTUNG
  unsigned int c_hfe;		// Verstrkungsfaktor Kollektorschaltung (Emitterfolger)
#endif

#ifdef R_MESS
  struct resis_t *thisR;
  unsigned long lrx1;
  unsigned long llrx1;
  unsigned long llrx2;
#endif
  /*
    HighPin wird fest auf Vcc gelegt
    LowPin wird ber R_L auf GND gelegt
    TristatePin wird hochohmig geschaltet, dafr ist keine Aktion ntig
  */
  wdt_reset();
#ifdef USE_EEPROM
  LoPinRL = eeprom_read_byte(&PinRLtab[LowPin]);	// Schaltbefehl fr LowPin R_L
  LoPinRH = LoPinRL + LoPinRL;				// Schaltbefehl for LowPin R_H
  TriPinRL = eeprom_read_byte(&PinRLtab[TristatePin]);	// Schaltbefehl fr TristatePin R_L
  TriPinRH = TriPinRL + TriPinRL;			// Schaltbefehl fr TristatePin R_H
  HiPinRL = eeprom_read_byte(&PinRLtab[HighPin]);	// Schaltbefehl fr HighPin R_L
  HiPinRH = HiPinRL + HiPinRL;				// Schaltbefehl fr HighPin R_H

  HiADCp = eeprom_read_byte(&PinADCtab[HighPin]);	// Schaltbefehl fr ADC High-Pin 
  LoADCp = eeprom_read_byte(&PinADCtab[LowPin]);		// Schaltbefehl fr ADC Low-Pin
#else
  LoPinRL = pgm_read_byte(&PinRLtab[LowPin]);		// Schaltbefehl fr LowPin R_L
  LoPinRH = LoPinRL + LoPinRL;				// Schaltbefehl for LowPin R_H
  TriPinRL = pgm_read_byte(&PinRLtab[TristatePin]);	// Schaltbefehl fr TristatePin R_L
  TriPinRH = TriPinRL + TriPinRL;			// Schaltbefehl fr TristatePin R_H
  HiPinRL = pgm_read_byte(&PinRLtab[HighPin]);		// Schaltbefehl fr HighPin R_L
  HiPinRH = HiPinRL + HiPinRL;				// Schaltbefehl fr HighPin R_H

  HiADCp = pgm_read_byte(&PinADCtab[HighPin]);		// Schaltbefehl fr ADC High-Pin 
  LoADCp = pgm_read_byte(&PinADCtab[LowPin]);		// Schaltbefehl fr ADC Low-Pin
#endif
  HiADCm = HiADCp | TXD_MSK;
  HiADCp |= TXD_VAL;
  LoADCm = LoADCp | TXD_MSK;
  LoADCp |= TXD_VAL;

  //Pins setzen
  R_PORT = 0;				//Widerstands-Port Ausgnge auf 0
  R_DDR = LoPinRL;			//Low-Pin auf Ausgang und ber R_L auf Masse
  ADC_DDR = HiADCm;		//High-Pin auf Ausgang
  ADC_PORT = HiADCp;		//High-Pin fest auf Vcc
  //Bei manchen MOSFETs muss das Gate (TristatePin) zuerst entladen werden
  ChargePin10ms(TriPinRL,0);		//entladen fr N-Kanal
  adc.lp_otr = W5msReadADC(LowPin);	//Spannung am Low-Pin ermitteln
  if(adc.lp_otr >= 977) {		//Sperrt das Bauteil jetzt?
     ChargePin10ms(TriPinRL,1);	 	//sonst: Entladen fr P-Kanal (Gate auf Plus)
     adc.lp_otr = ReadADC(LowPin);	//Spannung am Low-Pin erneut ermitteln
  }

#if DebugOut == 5
  Line2();
  lcd_clear_line();
  Line2();
#endif

  if(adc.lp_otr > 92) {  //Bauteil leitet ohne Steuerstrom etwas
#if DebugOut == 5
     lcd_ziff1(LowPin);
     lcd_data('F');
     lcd_ziff1(HighPin);
     lcd_data(' ');
     wait1s();
#endif
     //Test auf N-JFET oder selbstleitenden N-MOSFET
     R_DDR = LoPinRL | TriPinRH;	//Tristate-Pin (vermutetes Gate) ber R_H auf Masse
     adc.lp1 = W20msReadADC(LowPin);	//Spannung am vermuteten Source messen
     R_PORT = TriPinRH;			//Tristate-Pin (vermutetes Gate) ber R_H auf Plus
     adc.lp2 = W20msReadADC(LowPin);	//Spannung am vermuteten Source erneut messen
     //Wenn es sich um einen selbstleitenden MOSFET oder JFET handelt, msste adc.lp2 > adc.lp1 sein
     if(adc.lp2>(adc.lp1+488)) {
        //Spannung am Gate messen, zur Unterscheidung zwischen MOSFET und JFET
        ADC_PORT = TXD_VAL;
        ADC_DDR = LoADCm;	//Low-Pin fest auf Masse
        R_DDR = TriPinRH | HiPinRL;	//High-Pin auf Ausgang
        R_PORT = TriPinRH | HiPinRL;	//High-Pin ber R_L auf Vcc
        adc.lp2 = W20msReadADC(TristatePin); //Spannung am vermuteten Gate messen
        if(adc.lp2>3911) {  //MOSFET
           PartFound = PART_FET;	//N-Kanal-MOSFET
           PartMode = PART_MODE_N_D_MOS; //Verarmungs-MOSFET
        } else {  //JFET (pn-bergang zwischen G und S leitet)
           PartFound = PART_FET;	//N-Kanal-JFET
           PartMode = PART_MODE_N_JFET;
        }
        trans.b = TristatePin;
        trans.c = HighPin;
        trans.e = LowPin;
     }

     ADC_PORT = TXD_VAL;		// direkte Ausgnge auf Masse

     //Test auf P-JFET oder selbstleitenden P-MOSFET
     ADC_DDR = LoADCm;		//Low-Pin (vermuteter Drain) fest auf Masse,
					//Tristate-Pin (vermutetes Gate) ist noch ber R_H auf Plus
     R_DDR = TriPinRH | HiPinRL;	//High-Pin auf Ausgang
     R_PORT = TriPinRH | HiPinRL;	//High-Pin ber R_L auf Vcc
     adc.hp1 = W20msReadADC(HighPin);	//Spannung am vermuteten Source messen
     R_PORT = HiPinRL;			//Tristate-Pin (vermutetes Gate) ber R_H auf Masse
     adc.hp2 = W20msReadADC(HighPin);	//Spannung am vermuteten Source erneut messen
     //Wenn es sich um einen selbstleitenden P-MOSFET oder P-JFET handelt, msste adc.hp1 > adc.hp2 sein
     if(adc.hp1>(adc.hp2+488)) {
        //Spannung am Gate messen, zur Unterscheidung zwischen MOSFET und JFET
        ADC_PORT = HiADCp;	//High-Pin fest auf Plus
        ADC_DDR = HiADCm;	//High-Pin auf Ausgang
        adc.tp2 = W20msReadADC(TristatePin); //Spannung am vermuteten Gate messen
        if(adc.tp2<977) { 		//MOSFET
           PartFound = PART_FET;	//P-Kanal-MOSFET
           PartMode = PART_MODE_P_D_MOS; //Verarmungs-MOSFET
        } else { 			//JFET (pn-bergang zwischen G und S leitet)
           PartFound = PART_FET;	//P-Kanal-JFET
           PartMode = PART_MODE_P_JFET;
        }
        trans.b = TristatePin;
        trans.c = LowPin;
        trans.e = HighPin;
     }
  } // end Bauteil leitet etwas ohne Steuerstrom


#ifdef KOLLEKTOR_SCHALTUNG
  // Test Kollektorschaltung (Emitterfolger)
  ADC_PORT = TXD_VAL;
  ADC_DDR = LoADCm;			// Kollektor direkt auf GND
  R_PORT = HiPinRL;			// Emitter ber Widerstand R_L auf VCC
  R_DDR = TriPinRL | HiPinRL;		// Basiswiderstand R_L auf GND
  adc.hp1 = U_VCC - W5msReadADC(HighPin);	// Spannung am Emitterwiderstand
  adc.tp1 = ReadADC(TristatePin);	// BasiswiderstandsSpannung
  if (adc.tp1 < 10) {
     R_DDR =  TriPinRH | HiPinRL;	// Tripin=RH-
     adc.hp1 = U_VCC - W5msReadADC(HighPin);
     adc.tp1 = ReadADC(TristatePin);	// Spannung am Basiswiderstand
#ifdef LONG_HFE
     c_hfe = ((unsigned int)adc.hp1 * (unsigned long)(((unsigned long)R_H_VAL * 100) / 
               (unsigned int)(R_L_VAL+PIN_RP))) / (unsigned int)adc.tp1;	
#else
     c_hfe = ((adc.hp1 / ((R_L_VAL+PIN_RP+50)/100)) * (R_H_VAL/50)) / (adc.tp2/50);
#endif
  } else {
     c_hfe = (adc.hp1 - adc.tp1) / adc.tp1;
  }
#endif

  //Pins erneut setzen fr Emitterschaltung
  R_DDR = LoPinRL;			//Low-Pin auf Ausgang und ber R_L auf Masse
  R_PORT = 0;
  ADC_DDR = HiADCm;		//High-Pin auf Ausgang
  ADC_PORT = HiADCp;		//High-Pin fest auf Vcc
  wait5ms();
  
  if(adc.lp_otr < 977) {
     //Wenn das Bauteil keinen Durchgang zwischen HighPin und LowPin hat
#if DebugOut == 5
     lcd_ziff1(LowPin);
     lcd_data('P');
     lcd_ziff1(HighPin);
     lcd_data(' ');
     wait1s();
#endif
     //Test auf pnp
     R_DDR = LoPinRL | TriPinRL;	//Tristate-Pin ber R_L auf Masse, zum Test auf pnp
     adc.lp1 = W5msReadADC(LowPin);		//Spannung messen
     if(adc.lp1 > 3422) {
        //Bauteil leitet => pnp-Transistor o..
        //Verstrkungsfaktor in beide Richtungen messen
        R_DDR = LoPinRL | TriPinRH;	//Tristate-Pin (Basis) ber R_H auf Masse

        adc.lp1 = W5msReadADC(LowPin);	//Spannung am Low-Pin (vermuteter Kollektor) messen
        adc.tp2 = ReadADC(TristatePin);	//Basisspannung messen
        //Prfen, ob Test schon mal gelaufen
        if((PartFound == PART_TRANSISTOR) || (PartFound == PART_FET)) {
           PartReady = 1;
        }

        //Verstrkungsfaktor berechnen Emitterschaltung
        //hFE = Kollektorstrom / Basisstrom
        if(adc.tp2 < 53) {
#if DebugOut == 5
           lcd_data('<');
           lcd_data('5');
           lcd_data('3');
#endif
           adc.tp2 = 53;
        }

 #ifdef LONG_HFE
        trans.hfe[PartReady] = ((unsigned int)adc.lp1 * (unsigned long)(((unsigned long)R_H_VAL * 100) / 
                 (unsigned int)(R_L_VAL+PIN_RM))) / (unsigned int)adc.tp2;	
 #else
        trans.hfe[PartReady] = ((adc.lp1 / ((R_L_VAL+PIN_RM+50)/100)) * (R_H_VAL/50)) / (adc.tp2/50);
 #endif
#ifdef KOLLEKTOR_SCHALTUNG
        //Verstrkungsfaktor Kollektorschaltung (Emitterfolger)
        // hFE = (Emitterstrom - Basisstrom) / Basisstrom
        if (c_hfe > trans.hfe[PartReady]) {
           trans.hfe[PartReady] = c_hfe;
        }
#endif

 
        if(PartFound != PART_THYRISTOR) {
           if(adc.tp2 > 977) {
              //PNP-Transistor gefunden (Basis wird "nach oben" gezogen)
              PartFound = PART_TRANSISTOR;
              PartMode = PART_MODE_PNP;
           } else {
              if((adc.lp_otr < 97) && (adc.lp1 > 2000)) {
                 //Durchlassspannung im gesperrten Zustand gering genug?
                 //(sonst werden D-Mode-FETs flschlicherweise als E-Mode erkannt)
        	 PartFound = PART_FET;		//P-Kanal-MOSFET gefunden (Basis/Gate wird NICHT "nach oben" gezogen)
        	 PartMode = PART_MODE_P_E_MOS;
                 PinMSK = LoADCm & 7;
        	 //Messung der Gate-Schwellspannung
        	 ADMUX = TristatePin | (1<<REFS0);	// switch to TristatePin, Ref. VCC
        	 gthvoltage = 1;		// runden ((1*4)/9)
        	 for(ii=0;ii<11;ii++) {
        	    wdt_reset();
        	    ChargePin10ms(TriPinRL,1);
                    R_DDR = LoPinRL | TriPinRH;	//Tristate-Pin (Basis) ber R_H auf Masse
        	    while (!(ADC_PIN&PinMSK));  // Warten, bis der MOSFET schaltet und Drain auf high geht
        	    R_DDR = LoPinRL;
        	    ADCSRA |= (1<<ADSC);	// Start Conversion
        	    while (ADCSRA&(1<<ADSC));	// wait
      		    gthvoltage += (U_VCC - ADCW); // Add Tristatepin-Voltage
                 }
                 gthvoltage *= 4;		// entspricht 44*ADCW
                 gthvoltage /= 9;		// umrechnen in mV
              }
           }
           trans.b = TristatePin;
           trans.c = LowPin;
           trans.e = HighPin;
        }  // end if PartFound != PART_THYRISTOR
    } // end Bauteil leitet => pnp

#ifdef KOLLEKTOR_SCHALTUNG
    // Low-Pin=RL- HighPin=VCC
    R_DDR = LoPinRL | TriPinRL;
    R_PORT = TriPinRL;			// TriPin=RL+  Kollektorschaltung NPN
    adc.lp1 = W5msReadADC(LowPin);	// Spannung am Emitterwiderstand
    adc.tp1 = U_VCC - ReadADC(TristatePin);	// Spannung am Basiswiderstand
    if (adc.tp1 < 10) {
       R_DDR = LoPinRL | TriPinRH;
       R_PORT = TriPinRH;		// Tripin=RH+
       adc.lp1 = W5msReadADC(LowPin);
       adc.tp1 = U_VCC - ReadADC(TristatePin);	// Spannung am Basiswiderstand
#ifdef LONG_HFE
       c_hfe = ((unsigned int)adc.lp1 * (unsigned long)(((unsigned long)R_H_VAL * 100) / 
                 (unsigned int)(R_L_VAL+PIN_RM))) / (unsigned int)adc.tp1;	
#else
       c_hfe = ((adc.lp1 / ((R_L_VAL+PIN_RM+50)/100)) * (R_H_VAL/50)) / (adc.tp2/50);
#endif
    } else {
       c_hfe = (adc.lp1 - adc.tp1) / adc.tp1;
    }
#endif
    //Tristate (vermutete Basis) auf Plus, zum Test auf npn
    ADC_DDR = LoADCm;		//Low-Pin auf Ausgang 0V
    ADC_PORT = TXD_VAL;			//Low-Pin fest auf Masse
    R_DDR = TriPinRL | HiPinRL;		//High-Pin und Tristate-Pin auf Ausgang
    R_PORT = TriPinRL | HiPinRL;	//High-Pin und Tristate-Pin ber R_L auf Vcc
    adc.hp1 = W5msReadADC(HighPin);	//Spannung am High-Pin messen
    if(adc.hp1 < 1600) {
       //Bauteil leitet => npn-Transistor o..
#if DebugOut == 5
       lcd_ziff1(LowPin);
       lcd_data('N');
       lcd_ziff1(HighPin);
       lcd_data(' ');
       wait1s();
#endif
       if(PartReady==1) {
//          goto testend;
          goto widmes;
       }

       //Test auf Thyristor:
       //Gate entladen
       ChargePin10ms(TriPinRL,0);	//Tristate-Pin (Gate) ber R_L 10ms auf Masse
       adc.hp3 = W5msReadADC(HighPin);	//Spannung am High-Pin (vermutete Anode) erneut messen
					//Strom soll weiterflieen, falls nicht,
					// kein Thyristor oder Haltestrom zu niedrig
		 	
       R_PORT = 0;			//High-Pin (vermutete Anode) auf Masse (lschen)
       wait5ms();
       R_PORT = HiPinRL;		//High-Pin (vermutete Anode) wieder auf Plus
       adc.hp2 = W5msReadADC(HighPin);	//Spannung am High-Pin (vermutete Anode) erneut messen
       if((adc.hp3 < 1600) && (adc.hp2 > 4400)) {
          //Nach Abschalten des Haltestroms muss der Thyristor sperren
          //war vor Abschaltung des Triggerstroms geschaltet und ist immer noch geschaltet obwohl Gate aus => Thyristor
          PartFound = PART_THYRISTOR;
          //Test auf Triac
          R_DDR = 0;
          R_PORT = 0;
          ADC_PORT = LoADCp;	//Low-Pin fest auf Plus
          wait5ms();
          R_DDR = HiPinRL;		//HighPin ber R_L auf Masse
          if(W5msReadADC(HighPin) > 244) {
             goto savenresult;		//Spannung am High-Pin (vermuteter A2) messen; falls zu hoch:
          }
                                	//Bauteil leitet jetzt => kein Triac
          R_DDR = HiPinRL | TriPinRL;	//Gate auch ber R_L auf Masse => Triac msste znden
          if(W5msReadADC(TristatePin) < 977) {
             goto savenresult; 		//Spannung am Tristate-Pin (vermutetes Gate) messen;
                              		// Abbruch falls Spannung zu gering
          }
          if(ReadADC(HighPin) < 733) {
             goto savenresult; 		//Bauteil leitet jetzt nicht => kein Triac => Abbruch
          }
          R_DDR = HiPinRL;		//TristatePin (Gate) wieder hochohmig
          if(W5msReadADC(HighPin) < 733) {
             goto savenresult; 		//Bauteil leitet nach Abschalten des Gatestroms nicht mehr=> kein Triac => Abbruch
          }
          R_PORT = HiPinRL;		//HighPin ber R_L auf Plus => Haltestrom aus
          wait5ms();
          R_PORT = 0;			//HighPin wieder ber R_L auf Masse; Triac msste jetzt sperren
          if(W5msReadADC(HighPin) > 244) {
             goto savenresult;		//Spannung am High-Pin (vermuteter A2) messen;
                                	//falls zu hoch: Bauteil leitet jetzt => kein Triac
         }
         PartFound = PART_TRIAC;
         PartReady = 1;
         goto savenresult;
        }
      //Test auf Transistor oder MOSFET
      // ADC_DDR = LoADCm;	//Low-Pin auf Ausgang 0V
      R_DDR = HiPinRL | TriPinRH;	//Tristate-Pin (Basis) auf Ausgang
      R_PORT = HiPinRL | TriPinRH;	//Tristate-Pin (Basis) ber R_H auf Plus
      wait50ms();
      adc.hp2 = U_VCC - ReadADC(HighPin);	//Spannung am Kollektorwiderstand messen
      adc.tp2 = U_VCC - ReadADC(TristatePin);	//Spannung am Basiswiderstand messen

#if DebugOut == 5
       Line3();
       lcd_clear_line();
       Line3();
       lcd_data('H');
       lcd_data('P');
       lcd_string(utoa(adc.hp2,outval,10));
       lcd_data(' ');
       lcd_data('T');
       lcd_data('P');
       lcd_string(utoa(adc.tp2,outval,10));
       Line4();
       lcd_clear_line();
       Line4();
       lcd_data('L');
       lcd_data('P');
       lcd_string(utoa(adc.lp1,outval,10));
       lcd_data(' ');
       lcd_data('T');
       lcd_data('P');
       lcd_string(utoa(adc.tp1,outval,10));
       wait1s();
#endif
      if((PartFound == PART_TRANSISTOR) || (PartFound == PART_FET)) {
         PartReady = 1;	//prfen, ob Test schon mal gelaufen
      }

      //Verstrkungsfaktor berechnen Emitterschaltung
      //hFE = Kollektorstrom / Basisstrom
      if(adc.tp2 < 53) {
#if DebugOut == 5
         lcd_data('<');
         lcd_data('5');
         lcd_data('3');
#endif
         adc.tp2 = 53;
      }

#ifdef LONG_HFE
      trans.hfe[PartReady] = ((unsigned int)adc.hp2 * (unsigned long)(((unsigned long)R_H_VAL * 100) / 
              (unsigned int)(R_L_VAL+PIN_RP))) / (unsigned int)adc.tp2;	
#else
      trans.hfe[PartReady] = ((adc.hp2 / ((R_L_VAL+PIN_RP+50)/100)) * (R_H_VAL/50)) / (adc.tp2/50);
#endif
#ifdef KOLLEKTOR_SCHALTUNG
       //Verstrkungsfaktor vergleichen Kollektorschaltung (Emitterfolger)
       // hFE = (Emitterstrom - Basisstrom) / Basisstrom
       if (c_hfe >  trans.hfe[PartReady]) {
          trans.hfe[PartReady] = c_hfe;
       }
#endif

      if(adc.tp2 > 2557) {		// Basis-Spannung R_H klein genug
         PartFound = PART_TRANSISTOR;	//NPN-Transistor gefunden (Basis wird "nach unten" gezogen)
         PartMode = PART_MODE_NPN;
      } else { // Basis zieht kaum Strom
         if((adc.lp_otr < 97) && (adc.hp2 > 3400)) {
            //Durchlassspannung im gesperrten Zustand gering genug?
            //(sonst werden D-Mode-FETs flschlicherweise als E-Mode erkannt)
            PartFound = PART_FET;	//N-Kanal-MOSFET gefunden (Basis/Gate wird NICHT "nach unten" gezogen)
            PartMode = PART_MODE_N_E_MOS;
#if DebugOut == 5
            Line3();
            lcd_clear_line();
            Line3();
            lcd_data('N');
            lcd_data('F');
            wait1s();
#endif
            PinMSK = HiADCm & 7;
            //Gate-Schwellspannung messen
            ADMUX = TristatePin | (1<<REFS0);	// messe TristatePin, Ref. VCC
            gthvoltage = 1;			// runden ((1*4)/9)
            for(ii=0;ii<11;ii++) {
            	wdt_reset();
            	ChargePin10ms(TriPinRL,0);	// Gate 10ms mit RL entladen
                R_DDR = HiPinRL | TriPinRH;	// Gate langsam laden
                R_PORT = HiPinRL | TriPinRH;
            	while ((ADC_PIN&PinMSK));  // Warten, bis der MOSFET schaltet und Drain auf low geht
                R_DDR = HiPinRL;
            	ADCSRA |= (1<<ADSC);		// starte Conversion
            	while (ADCSRA&(1<<ADSC));	// warte
            	gthvoltage += ADCW;
#if DebugOut == 5
                lcd_data('.');
#endif
            }
            gthvoltage *= 4;	//entspricht 44 * ADCW
            gthvoltage /= 9;	//umrechnen in mV
         }
      }
savenresult:
      trans.b = TristatePin;		// merke Pin-Konstellation
      trans.c = HighPin;
      trans.e = LowPin;
   } // end Bauteil leitet => npn
   ADC_DDR = TXD_MSK;		// alle ADC-Ports hochohmig
   ADC_PORT = TXD_VAL;		// alls ADC-Ports auf 0 (kein Pullup)
   //Fertig
   //end das Bauteil hat keinen Durchgang zwischen HighPin und LowPin
   goto widmes;
  }
  // Bauteil hat Durchgang
  //Test auf Diode
  ADC_PORT = TXD_VAL;
  ADC_DDR = LoADCm;	//Low-Pin fest auf Masse
  R_DDR = HiPinRL;		//High-Pin ber R_L auf Plus
  R_PORT = HiPinRL;
  ChargePin10ms(TriPinRL,1);	//Entladen fr P-Kanal-MOSFET
  adc.lp_otr = W5msReadADC(HighPin) - ReadADC(LowPin);
  R_DDR = HiPinRH;		//High-Pin ber R_H auf Plus
  R_PORT = HiPinRH;
  adc.hp2 = W5msReadADC(HighPin); 		// M--|<--HP--R_H--Plus

  R_DDR = HiPinRL;		//High-Pin ber R_L auf Plus
  R_PORT = HiPinRL;
  ChargePin10ms(TriPinRL,0);	//Entladen fr N-Kanal-MOSFET
  adc.hp1 = W5msReadADC(HighPin) - W5msReadADC(LowPin);
  R_DDR = HiPinRH;		//High-Pin ber R_H  auf Plus
  R_PORT = HiPinRH;
  adc.hp3 = W5msReadADC(HighPin);		// M--|<--HP--R_H--Plus

    /*Ohne das Entladen kann es zu Falscherkennungen kommen, da das Gate eines MOSFETs noch geladen sein kann.
      Die zustzliche Messung mit dem "groen" Widerstand R_H wird durchgefhrt, um antiparallele Dioden von
      Widerstnden unterscheiden zu knnen.
      Eine Diode hat eine vom Durchlassstrom relativ unabhngige Durchlassspg.
      Bei einem Widerstand ndert sich der Spannungsabfall stark (linear) mit dem Strom.
    */
  if(adc.lp_otr > adc.hp1) {
      adc.hp1 = adc.lp_otr;	//der hhere Wert gewinnt
      adc.hp3 = adc.hp2;
  }
#if DebugOut == 4
  Line3();
  lcd_clear_line();
  Line3();
  lcd_ziff1(HighPin);
  lcd_data('D');
  lcd_ziff1(LowPin);
  lcd_data(' ');
  lcd_data('L');
  lcd_string(utoa(adc.hp1,outval,10));
  lcd_data(' ');
  lcd_data('H');
  lcd_string(utoa(adc.hp3,outval,10));
  lcd_data(' ');
  wait1s();
#endif

  if((adc.hp1 > 150) && (adc.hp1 < 4640) && (adc.hp1 > (adc.hp3+(adc.hp3/8))) && (adc.hp3*8 > adc.hp1)) {
     //Spannung liegt ber 0,15V und unter 4,64V => Ok
     if((PartFound == PART_NONE) || (PartFound == PART_RESISTOR)) {
        PartFound = PART_DIODE;	//Diode nur angeben, wenn noch kein anderes Bauteil gefunden wurde.
				//Sonst gbe es Probleme bei Transistoren mit Schutzdiode
#if DebugOut == 4
        lcd_data('D');
#endif
     }
     diodes[NumOfDiodes].Anode = HighPin;
     diodes[NumOfDiodes].Cathode = LowPin;
     diodes[NumOfDiodes].Voltage = adc.hp1;	// Spannung in Millivolt 
     NumOfDiodes++;
#if 0
     for(ii=0;ii<NumOfDiodes;ii++) {
        if((diodes[ii].Anode == LowPin) && (diodes[ii].Cathode == HighPin)) {
           //zwei antiparallele Dioden: Defekt oder Duo-LED
           if((adc.hp3*6) < (adc.hp1 / 53)) { 
              //Durchlassspannung fllt bei geringerem Teststrom stark ab => Defekt
              if(ii<NumOfDiodes) {
       	         for(uint8_t j=ii;j<(NumOfDiodes-1);j++) {
       	            diodes[j].Anode = diodes[j+1].Anode;
       	            diodes[j].Cathode = diodes[j+1].Cathode;
       	            diodes[j].Voltage = diodes[j+1].Voltage;
       	         }
              }
 #if DebugOut == 4
              lcd_data('R');
              lcd_ziff1(NumOfDiodes);
 #endif
              NumOfDiodes -= 2;
           }
        }
     } // end for
#endif
  } //end Spannung liegt ber 0,15V und unter 4,64V 
#if DebugOut == 4
  lcd_data(NumOfDiodes+'0');
#endif

widmes:
#ifdef R_MESS	//Widerstandsmessung nur auf dem Mega8 verfgbar
  //Test auf Widerstand
  wdt_reset();
  ADC_PORT = TXD_VAL;
  ADC_DDR = LoADCm;	//Low-Pin fest auf Masse
  R_DDR = HiPinRL;		//High-Pin ber R_L auf Plus
  R_PORT = HiPinRL;	
//  adc.lp1 = W5msReadADC(LowPin);			// low-Spannung unter Last
//  adc.hp1 = ReadADC(HighPin) - adc.lp1;	// Spannung am Widerstand RX mit R_L
//  adc.hp2 = ReadADC(HighPin) - adc.lp1;	// Spannung am Widerstand RX mit R_L
  adc.hp1 = ReadADC(HighPin);
  R_PORT = 0;
  R_DDR = HiPinRH;		//High-Pin ber R_H auf Masse
  adc.hp2 = W5msReadADC(HighPin);
  if (adc.hp2 > 20) {
     // Bei Widerstand sollte Spannung verschwinden
 #if DebugOut == 3
     Line3();
     lcd_clear_line();
     Line3();
     lcd_ziff1(LowPin);
     lcd_data('U');
     lcd_ziff1(HighPin);
     lcd_data('A');
     lcd_string(utoa(adc.hp1, outval, 10));
     lcd_data('B');
     lcd_string(utoa(adc.hp2, outval, 10));
     lcd_data(' ');
 #endif
     goto testend;
  }
  R_PORT = HiPinRH;		//High-Pin ber R_H auf Plus
  adc.hp2 = W5msReadADC(HighPin);			// Spannung am Widerstand RX mit R_H

  //Messung der Spannungsdifferenz zwischen dem Pluspol von R_L und Vcc
//  ADC_DDR = HiADCm;	//High-Pin auf Ausgang
//  ADC_PORT = HiADCp;	//High-Pin fest auf Plus
//  R_PORT = 0;
//  R_DDR = LoPinRL;				//Low-Pin ber R_L auf Masse
//  adc.hp3 = W5msReadADC(HighPin) - adc.lp1;	//U_VCC - Spannungs-Abfall unter RX+R_L Last
  ADC_DDR = HiADCm;	//High-Pin auf Ausgang
  ADC_PORT = HiADCp;	//High-Pin fest auf Plus
  R_PORT = 0;
  R_DDR = LoPinRL;				//Low-Pin ber R_L auf Masse
  adc.lp1 = W5msReadADC(LowPin);
  R_DDR = LoPinRH;				//Low-Pin ber R_H auf Masse
  adc.lp2 = W5msReadADC(LowPin);
#if DebugOut == 3
  Line3();
  lcd_clear_line();
  Line3();
  lcd_ziff1(LowPin);
  lcd_data('R');
  lcd_ziff1(HighPin);
  lcd_data('L');
  lcd_string(utoa(adc.hp1, outval, 10));
  lcd_data('H');
  lcd_string(utoa(adc.hp2, outval, 10));
  Line4();
  lcd_clear_line();
  Line4();
  lcd_data('L');
  lcd_string(utoa(adc.lp1, outval, 10));
  lcd_data('H');
  lcd_string(utoa(adc.lp2, outval, 10));
  Line2();
#endif
		
  if((adc.hp1 < 4400) && (adc.hp2 > 97)) {
     //Spannung fllt bei geringem Teststrom nicht weit genug ab
 #if DebugOut == 3
     lcd_data('F');
 #endif
     goto testend; 
  }
//    if((adc.hp2 + (adc.hp2 / 61)) < adc.hp1)
  if (adc.hp2 < 4972) { 
     //Abfallende Spannung fllt bei geringerem Teststrom stark ab und es besteht kein "Beinahe-Kurzschluss" => Widerstand
     if (adc.hp1 > 4821) { // lieber Messung mit R_H nehmen
        if (adc.lp2 < 40) {
           // Messung > 50MOhm unplausible
           goto testend;
        }
        llrx1 = (unsigned long)R_H_VAL * (unsigned long)adc.hp2 / (U_VCC - adc.hp2);
        llrx2 = (unsigned long)R_H_VAL * (unsigned long)(U_VCC - adc.lp2) / adc.lp2;
#define U_INT_LIMIT 990		// 1V Schaltgrenze in ReadADC mit Sicherheitsabstand
#ifdef __AVR_ATmega8__
#define FAKT_LOW 2		//etwa doppelt so hohe Auflsung
#else
#define FAKT_LOW 4		//etwa viermal so hohe Auflsung
#endif
#ifdef AUTOSCALE_ADC
        if (adc.hp2 < U_INT_LIMIT) {
           lrx1 = (llrx1*FAKT_LOW + llrx2) / (FAKT_LOW+1);	//gewichteter Mittelwert zwischen beiden R_H Messungen
        } else if (adc.lp2 < U_INT_LIMIT){
           lrx1 = (llrx2*FAKT_LOW + llrx1) / (FAKT_LOW+1);	//gewichteter Mittelwert zwischen beiden R_H Messungen
        } else 
#endif
        {
           lrx1 = (llrx1 + llrx2) / 2;		//Mittelwert zwischen beiden R_H Messungen
        }
        lrx1 *= 100;
     } else {
//        lrx1 =(unsigned long)R_L_VAL * (unsigned long)adc.hp1 / (adc.hp3 - adc.hp1);
        llrx1 =(unsigned long)(R_L_VAL+PIN_RP) * (unsigned long)adc.hp1 / (U_VCC - adc.hp1);
        if (llrx1 > PIN_RM) {
           llrx1 -= PIN_RM;
        } else {
           llrx1 =0;			// auf 0 Ohm begrenzen
        }
        llrx2 =(unsigned long)(R_L_VAL+PIN_RM) * (unsigned long)(U_VCC -adc.lp1) / adc.lp1;
        if (llrx2 > PIN_RP) {
           llrx2 -= PIN_RP;
        } else {
           llrx2 = 0;			// auf 0 Ohm begrenzen
        }
#ifdef AUTOSCALE_ADC
        if (adc.hp1 < U_INT_LIMIT) {
           lrx1 = (llrx1*FAKT_LOW + llrx2) / (FAKT_LOW+1);	//gewichteter Mittelwert zwischen beiden R_L Messungen
        } else if (adc.lp1 < U_INT_LIMIT) {
           lrx1 = (llrx2*FAKT_LOW + llrx1) / (FAKT_LOW+1);	//gewichteter Mittelwert zwischen beiden R_L Messungen
        } else
#endif
        {
           lrx1 = (llrx1 + llrx2) / 2;		//Mittelwert zwischen beiden R_L Messungen
        }
     }
#if DebugOut == 33
  Line4();
  lcd_clear_line();
  Line4();
  lcd_data('X');
  lcd_string(ultoa(llrx1, outval, 10));
  lcd_data('Y');
  lcd_string(ultoa(llrx2, outval, 10));
  lcd_data(' ');
  Line2();
#endif
     if((PartFound == PART_DIODE) || (PartFound == PART_NONE) || (PartFound == PART_RESISTOR)) {
        for (ii=0; ii<NumOfR; ii++) {
           // nach Messung mit umgekehrter Polaritt suchen
           thisR = &resis[ii];
           if (thisR->rt != TristatePin) {
              continue;
           }
           // muss die Messung mit umgekehrter Polaritt sein
           if ((labs((long)(lrx1 - thisR->rx))*10)/ (lrx1 + thisR->rx + 20)  > 0) {
 #if DebugOut == 3
              lcd_data('R');
              lcd_data('!');
              lcd_data('=');
 #endif
              goto testend; // <20% Abweichung
           }
           PartFound = PART_RESISTOR;
           goto testend;
        } // end for
        // kein gemessenen R mit gleichem Tristate-Pin gefunden, neu
        thisR = &resis[NumOfR];
        thisR->rx = lrx1;
        thisR->ra = LowPin;
        thisR->rb = HighPin;
        thisR->rt = TristatePin;
        NumOfR++;
#if DebugOut == 3
        lcd_data(NumOfR+'0');
        lcd_data('R');
#endif
     }
  }
#endif
  testend:
#ifdef DebugOut
#if DebugOut < 10
  wait2s();
#endif
#endif
  ADC_DDR = TXD_MSK;		// alle ADC-Pins Input
  ADC_PORT = TXD_VAL;		// alle ADC Ausgnge auf Ground, keine Pullup
  R_DDR = 0;			// alle Widerstands-Ausgnge auf Input
  R_PORT = 0;			// alle Widerstands-Ausgnge auf Ground, keine Pullup
} // end CheckPins()

