#include <avr/io.h>
#include "config.h"
#include <stdlib.h>
#include "wait1000ms.h"
#include "lcd-routines.h"

extern uint8_t minmul;
extern uint8_t mindiv;

// ANZ_MES = 44 fr hchste Genauigkeit, Summe von 44 einzelnen Messungen
// ANZ_MES = 22 fr mittlere Genauigkeit, 22 einzelne Messungen, Ergebnis wird zustzlich mit 2 multipliziert
// ANZ_MES = 11 fr hchste Geschwindigkeit, 11 einzelne Messungen, Ergebnis wird mit 4 multipliziert
// Durch die anschliessende Division durch 9 wird jeweils eine mV Auflsung erreicht.

#ifndef ANZ_MES
#define ANZ_MES 44
#endif

unsigned int ReadADC(uint8_t mux) {
  //ADC-Wert des angegebenen Kanals auslesen und als unsigned int zurckgeben
  unsigned int adcx;
  uint8_t jj;

  ADMUX = mux | (1<<REFS0);
  adcx = (ANZ_MES/11); 	// runden des Ergebnis (+4)
  ADCSRA |= (1<<ADSC);		//starte Conversion
  while (ADCSRA&(1<<ADSC));		//warte
  wait300us();

  for (jj = 0; jj < ANZ_MES; jj++) {
     //ANZ_MESS Messungen; fr bessere Genauigkeit
     ADCSRA |= (1<<ADSC);		//starte Conversion
     while (ADCSRA & (1<<ADSC));		//warte
     adcx += ADCW;		// 1. Messung verwerfen
#ifdef __AVR_ATmega8__
#define U_GRENZ 2048		//etwa 2V
#else 
#define U_GRENZ 1024		//etwa 1V
#endif
#ifdef AUTOSCALE_ADC
     if ((jj == 4) && (adcx < U_GRENZ)){
       goto lowADC;
     }
#endif
  }
#if ANZ_MES == 22
  adcx *= 2;
#endif
  
#if ANZ_MES == 11
  adcx *= 4;
#endif
  return adcx/9;

#ifdef AUTOSCALE_ADC
lowADC:
#if 0
  Line4();
  lcd_string(utoa(adcx, outval, 10));
  lcd_data('~');
#endif
  
  adcx = 0;
  ADMUX = mux | (1<<REFS1) | (1<<REFS0); // Internal 1.1V Referenz
  ADCSRA |= (1<<ADSC);		//starte Conversion
  while (ADCSRA&(1<<ADSC));		//warte
  wait5ms();
  wait1ms();
  for (jj=0; jj<minmul; jj++) {
     ADCSRA |= (1<<ADSC);		//starte Conversion
     while (ADCSRA&(1<<ADSC));		//warte
     adcx += ADCW;			//Messungen zusammenzhlen
  }
#if 0
  lcd_string(utoa(adcx, outval, 10));
  lcd_data('/');
  lcd_string(utoa(mindiv, outval, 10));
  lcd_data(' ');
  wait1s();
#endif
  return (adcx/mindiv);			//Auflsung mV
#endif
}

unsigned int W20msReadADC(uint8_t mux) {
   wait20ms();
   return (ReadADC(mux));
  }
  

unsigned int W5msReadADC(uint8_t mux) {
   wait5ms();
   return (ReadADC(mux));
  }

